package org.bdware.client;

import com.google.gson.Gson;
import org.zz.gmhelper.SM2KeyPair;

import java.io.*;

public class ContractRequest extends SM2Verifible
        implements Serializable, Comparable<ContractRequest> {

    private static final long serialVersionUID = 5516141428163407726L;
    public long hasValue;
    public long gasLimit;
    public Boolean needSeq = false; // not requestAll do not need sequent
    public int seq = 0; // only requestAll type contract need
    public boolean withDynamicAnalysis = false;
    public boolean withEvaluatesAnalysis = false;
    public String fromContract;
    String contractID;
    String contractDOI;
    // requester = reqPubKey
    String requester;
    // DOI of requester
    String requesterDOI;
    // action = rsaPrivKeyEncoded(aesKey);
    String action;
    // arg = aesKeyEncoded({action,arg});
    String arg;
    String requestID;
    boolean fromDebug = false;

    public static void main(String[] args) {
        ContractRequest req = new ContractRequest();
        // req.requester = rsa.toBase64Pubkey();
        req.contractID = "123";
        req.action = "main";
        req.arg = "arg1";
        String key =
                "{\"privateKey\":\"589d94ee5688358a1c5c18430dd9c75097ddddebf769f139da36a807911d20f8\",\"publicKey\":\"04d1924329f72ced148f6f333fb985ccbaa31b1e3aacf10be5f43d4a4ff5ad88899a005e79e37fc06993e1d66ada8cf8b711cb36f59538bb7d3e39e70fa9360ddd\"}";
        SM2KeyPair pair = SM2KeyPair.fromJson(key);
        req.doSignature(pair);
        System.out.println(req.verifySignature());
        System.out.println(new Gson().toJson(req));
        System.out.println(req.verifySignature());
    }

    public static ContractRequest parse(byte[] content) {
        try {
            ObjectInputStream input = new ObjectInputStream(new ByteArrayInputStream(content));
            return (ContractRequest) input.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public String getContractID() {
        return contractID;
    }

    public ContractRequest setContractID(String id) {
        contractID = id;
        return this;
    }

    public String getContractDOI() {
        return contractDOI;
    }

    public void setContractDOI(String doi) {
        contractDOI = doi;
    }

    public String getRequester() {
        return requester;
    }

    public void setRequester(String requester) {
        this.requester = requester;
    }

    public String getRequesterDOI() {
        if (requesterDOI == null || requesterDOI.equals("")) return "empty";
        return requesterDOI;
    }

    public void setRequesterDOI(String requesterDOI) {
        this.requesterDOI = requesterDOI;
    }

    public String getArg() {
        return arg;
    }

    public ContractRequest setArg(String arg) {
        this.arg = arg;
        return this;
    }

    public String getAction() {
        return action;
    }

    public ContractRequest setAction(String action) {
        this.action = action;
        return this;
    }

    public long getValue() {
        return hasValue;
    }

    public ContractRequest setValue(long hasValue) {
        this.hasValue = hasValue;
        return this;
    }

    public long getGasLimit() {
        return gasLimit;
    }

    public void setGasLimit(long gasLimit) {
        this.gasLimit = gasLimit;
    }

    public void setFromDebug(boolean fromDebug) {
        this.fromDebug = fromDebug;
    }

    public boolean fromDebug() {
        return fromDebug;
    }

    @Override
    public String getPublicKey() {
        return requester;
    }

    @Override
    public void setPublicKey(String pubkey) {
        setRequester(pubkey);
    }

    @Override
    public String getContentStr() {
        return String.format(
                "%s|%s|%s%s|%s",
                (contractDOI != null && contractDOI.length() > 0) ? contractDOI : contractID,
                action,
                arg,
                gasLimit > 0 ? "|" + gasLimit : "",
                requester);
    }

    public byte[] toByte() {
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            ObjectOutputStream bo = new ObjectOutputStream(out);
            bo.writeObject(this);
            return out.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public String getRequestID() {
        return requestID;
    }

    public void setRequestID(String str) {
        this.requestID = str;
    }

    @Override
    public int compareTo(ContractRequest cr) {
        return Integer.compare(seq, cr.seq);
    }
}
